/*
 * SerialMousekeyboardControl.h - Serial Mouse Keyboard Control V1.0
 * Copyright (C) 20024 Rodrigo Santos - Bit a Bit
 *
 * It is a library that allows you to control the mouse and keyboard
 * using serial communication.
 *For this, it is necessary to use a computer program, responsible for
 *receiving, interpreting the commands and sending the commands to the operating system.
 *
 *Licensed under the MIT License:
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in all
 *copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *SOFTWARE.
*/

//Ensure this library description is only included once
#ifndef SerialMouseKeyboardControl_h
#define SerialMouseKeyboardControl_h

#include <Arduino.h>

enum Device
{
    MOUSE,
    KEYBOARD
};

enum MouseEvent
{
    RIGHT_PRESS,
    RIGHT_RELEASE,
    LEFT_PRESS,
    LEFT_RELEASE,
    MIDDLE_PRESS,
    MIDDLE_RELEASE,
    SCROLL,
    POSITION,
    MOVE
};

enum KeyboardEvent
{
    PRESS,
    RELEASE
};

enum Key
{
    VK_BACK = 0x08,                 //BACKSPACE key
    VK_TAB = 0x09,                  //TAB key

    VK_CLEAR = 0x0C,                //CLEAR key
    VK_RETURN = 0x0D,               //ENTER key

    VK_SHIFT = 0x10,                //SHIFT key
    VK_CONTROL = 0x11,              //CTRL key
    VK_MENU = 0x12,                 //ALT key
    VK_PAUSE = 0x13,                //PAUSE key
    VK_CAPITAL = 0x14,              //CAPS LOCK key
    VK_KANA = 0x15,                 //Kana IME mode
    VK_HANGEUL = 0x15,              //Hangeul IME mode
    VK_HANGUL = 0x15,               //Hangul IME mode
    VK_IME_ON = 0x16,               //IME enabled
    VK_JUNJA = 0x17,                //Junja IME mode
    VK_FINAL = 0x18,                //Final IME mode
    VK_HANJA = 0x19,                //Hanja IME mode
    VK_KANJI = 0x19,                //Kangi IME mode
    VK_IME_OFF = 0x1A,              //IME disabled
    VK_ESCAPE = 0x1B,               //ESC key
    VK_CONVERT = 0x1C,              //Conversion to IME
    VK_NONCONVERT = 0x1D,           //Non-conversion to IME
    VK_ACCEPT = 0x1E,               //Acceptance of EMI
    VK_MODECHANGE = 0x1F,           //IME Mode Change Request
    VK_SPACE = 0x20,                //SPACEBAR key
    VK_PRIOR = 0x21,                //PAGE UP key
    VK_NEXT = 0x22,                 //PAGE DOWN key
    VK_END = 0x23,                  //END key
    VK_HOME = 0x24,                 //HOME key
    VK_LEFT = 0x25,                 //LEFT ARROW key
    VK_UP = 0x26,                   //UP ARROW key
    VK_RIGHT = 0x27,                //RIGHT ARROW key
    VK_DOWN = 0x28,                 //DOWN ARROW key
    VK_SELECT = 0x29,               //SELECT key
    VK_PRINT = 0x2A,                //PRINT key
    VK_EXECUTE = 0x2B,              //EXECUTE key
    VK_SNAPSHOT = 0x2C,             //PRINT SCREEN key
    VK_INSERT = 0x2D,               //INSERT key
    VK_DELETE = 0x2E,               //DELETE key
    VK_HELP = 0x2F,                 //HELP key
    KEY_0 = 0x30,                   //0 key
    KEY_1 = 0x31,                   //1 key
    KEY_2 = 0x32,                   //2 key
    KEY_3 = 0x33,                   //3 key
    KEY_4 = 0x34,                   //4 key
    KEY_5 = 0x35,                   //5 key
    KEY_6 = 0x36,                   //6 key
    KEY_7 = 0x37,                   //7 key
    KEY_8 = 0x38,                   //8 key
    KEY_9 = 0x39,                   //9 key

    KEY_A = 0x41,                   //A key
    KEY_B = 0x42,                   //B key
    KEY_C = 0x43,                   //C key
    KEY_D = 0x44,                   //D key
    KEY_E = 0x45,                   //E key
    KEY_F = 0x46,                   //F key
    KEY_G = 0x47,                   //G key
    KEY_H = 0x48,                   //H key
    KEY_I = 0x49,                   //I key
    KEY_J = 0x4A,                   //J key
    KEY_K = 0x4B,                   //K key
    KEY_L = 0x4C,                   //L key
    KEY_M = 0x4D,                   //M key
    KEY_N = 0x4E,                   //N key
    KEY_O = 0x4F,                   //O key
    KEY_P = 0x50,                   //P key
    KEY_Q = 0x51,                   //Q key
    KEY_R = 0x52,                   //R key
    KEY_S = 0x53,                   //S key
    KEY_T = 0x54,                   //T key
    KEY_U = 0x55,                   //U key
    KEY_V = 0x56,                   //V key
    KEY_W = 0x57,                   //W key
    KEY_X = 0x58,                   //X key
    KEY_Y = 0x59,                   //Y key
    KEY_Z = 0x5A,                   //Z key
    VK_LWIN = 0x5B,                 //WINDOWS LEFT Key
    VK_RWIN = 0x5C,                 //WINDOWS RIGHT key
    VK_APPS = 0x5D,                 //KEY APPLICATIONS

    VK_SLEEP = 0x5F,                //COMPUTER SLEEP key
    VK_NUMPAD0 = 0x60,              //NUMERIC KEYPAD key 0
    VK_NUMPAD1 = 0x61,              //NUMERIC KEYPAD key 1
    VK_NUMPAD2 = 0x62,              //NUMERIC KEYPAD key 2
    VK_NUMPAD3 = 0x63,              //NUMERIC KEYPAD key 3
    VK_NUMPAD4 = 0x64,              //NUMERIC KEYPAD key 4
    VK_NUMPAD5 = 0x65,              //NUMERIC KEYPAD key 5
    VK_NUMPAD6 = 0x66,              //NUMERIC KEYPAD key 6
    VK_NUMPAD7 = 0x67,              //NUMERIC KEYPAD key 7
    VK_NUMPAD8 = 0x68,              //NUMERIC KEYPAD key 8
    VK_NUMPAD9 = 0x69,              //NUMERIC KEYPAD key 9
    VK_MULTIPLY = 0x6A,             //MULTIPLY key
    VK_ADD = 0x6B,                  //ADD key
    VK_SEPARATOR = 0x6C,            //SEPARATOR key
    VK_SUBTRACT = 0x6D,             //SUBTRACT key
    VK_DECIMAL = 0x6E,              //DECIMAL key
    VK_DIVIDE = 0x6F,               //DIVIDE key
    VK_F1 = 0x70,                   //F1 key
    VK_F2 = 0x71,                   //F2 key
    VK_F3 = 0x72,                   //F3 key
    VK_F4 = 0x73,                   //F4 key
    VK_F5 = 0x74,                   //F5 key
    VK_F6 = 0x75,                   //F6 key
    VK_F7 = 0x76,                   //F7 key
    VK_F8 = 0x77,                   //F8 key
    VK_F9 = 0x78,                   //F9 key
    VK_F10 = 0x79,                  //F10 key
    VK_F11 = 0x7A,                  //F11 key
    VK_F12 = 0x7B,                  //F12 key
    VK_F13 = 0x7C,                  //F13 key
    VK_F14 = 0x7D,                  //F14 key
    VK_F15 = 0x7E,                  //F15 key
    VK_F16 = 0x7F,                  //F16 key
    VK_F17 = 0x80,                  //F17 key
    VK_F18 = 0x81,                  //F18 key
    VK_F19 = 0x82,                  //F19 key
    VK_F20 = 0x83,                  //F20 key
    VK_F21 = 0x84,                  //F21 key
    VK_F22 = 0x85,                  //F22 key
    VK_F23 = 0x86,                  //F23 key
    VK_F24 = 0x87,                  //F24 key

    VK_NUMLOCK = 0x90,              //NUM LOCK key
    VK_SCROLL = 0x91,               //SCROLL LOCK key
    VK_OEM_NEC_EQUAL = 0x92,        //OEM Specific
    VK_OEM_FJ_JISHO = 0x92,         //OEM Specific
    VK_OEM_FJ_MASSHOU = 0x93,       //OEM Specific
    VK_OEM_FJ_TOUROKU = 0x94,       //OEM Specific
    VK_OEM_FJ_LOYA = 0x95,          //OEM Specific
    VK_OEM_FJ_ROYA = 0x96,          //OEM Specific

    VK_LSHIFT = 0xA0,               //LEFT SHIFT key
    VK_RSHIFT = 0xA1,               //RIGHT SHIFT key
    VK_LCONTROL = 0xA2,             //LEFT CONTROL key
    VK_RCONTROL = 0xA3,             //RIGHT CONTROL key
    VK_LMENU = 0xA4,                //ALT LEFT key
    VK_RMENU = 0xA5,                //ALT RIGHT key
    VK_BROWSER_BACK = 0xA6,         //BROWNSER BACK key
    VK_BROWSER_FORWARD = 0xA7,      //BROWNSER FOWARD key
    VK_BROWSER_REFRESH = 0xA8,      //BROWNSER REFRESH key
    VK_BROWSER_STOP = 0xA9,         //BROWNSER STOP key
    VK_BROWSER_SEARCH = 0xAA,       //BROWNSER SEARCH key
    VK_BROWSER_FAVORITES = 0xAB,    //BROWNSER FAVORITES KEY
    VK_BROWSER_HOME = 0xAC,         //BROWNSER HOME key
    VK_VOLUME_MUTE = 0xAD,          //VOLUME MUTE key
    VK_VOLUME_DOWN = 0xAE,          //VOLUME DOWN key
    VK_VOLUME_UP = 0xAF,            //VOLUME UP key
    VK_MEDIA_NEXT_TRACK = 0xB0,     //NEXT TRACK key
    VK_MEDIA_PREV_TRACK = 0xB1,     //PREVIOUS TRACK key
    VK_MEDIA_STOP = 0xB2,           //MEDIA STOP key
    VK_MEDIA_PLAY_PAUSE = 0xB3,     //MEDIA PLAY/PAUSE key
    VK_LAUNCH_MAIL = 0xB4,          //LAUNCH MAIL key
    VK_LAUNCH_MEDIA_SELECT = 0xB5,  //LAUNCH MEDIA SELECT key
    VK_LAUNCH_APP1 = 0xB6,          //LAUNCH APPLICATION 1 key
    VK_LAUNCH_APP2 = 0xB7,          //LAUNCH APPLICATION 2 key

    VK_OEM_1 = 0xBA,                //Used for miscellaneous characters. May vary by keyboard layout
    VK_OEM_PLUS = 0xBB,             //For any country/region, the + key
    VK_OEM_COMMA = 0xBC,            //For any country/region, the , key
    VK_OEM_MINUS = 0xBD,            //For any country/region, the - key
    VK_OEM_PERIOD = 0xBE,           //For any country/region, the . key
    VK_OEM_2 = 0xBF,                //Used for miscellaneous characters. May vary by keyboard layout
    VK_OEM_3 = 0xC0,                //Used for miscellaneous characters. May vary by keyboard layout

    VK_OEM_4 = 0xDB,                //Used for miscellaneous characters. May vary by keyboard layout
    VK_OEM_5 = 0xDC,                //Used for miscellaneous characters. May vary by keyboard layout
    VK_OEM_6 = 0xDD,                //Used for miscellaneous characters. May vary by keyboard layout
    VK_OEM_7 = 0xDE,                //Used for miscellaneous characters. May vary by keyboard layout
    VK_OEM_8 = 0xDF,                //Used for miscellaneous characters. May vary by keyboard layout

    VK_OEM_AX = 0xE1,               //OEM Specific
    VK_OEM_102 = 0xE2,              //Used for miscellaneous characters. May vary by keyboard layout
    VK_ICO_HELP = 0xE3,             //OEM Specific
    VK_ICO_00 = 0xE4,               //OEM Specific
    VK_PROCESSKEY = 0xE5,           //PROCESS key of the IME
    VK_ICO_CLEAR = 0xE6,            //OEM Specific
    VK_PACKET = 0xE7,               //Used to send Unicode characters directly

    VK_OEM_RESET = 0xE9,            //OEM Specific
    VK_OEM_JUMP = 0xEA,             //OEM Specific
    VK_OEM_PA1 = 0xEB,              //OEM Specific
    VK_OEM_PA2 = 0xEC,              //OEM Specific
    VK_OEM_PA3 = 0xED,              //OEM Specific
    VK_OEM_WSCTRL = 0xEE,           //OEM Specific
    VK_OEM_CUSEL = 0xEF,            //OEM Specific
    VK_OEM_ATTN = 0xF0,             //OEM Specific
    VK_OEM_FINISH = 0xF1,           //OEM Specific
    VK_OEM_COPY = 0xF2,             //OEM Specific
    VK_OEM_AUTO = 0xF3,             //OEM Specific
    VK_OEM_ENLW = 0xF4,             //OEM Specific
    VK_OEM_BACKTAB = 0xF5,          //OEM Specific
    VK_ATTN = 0xF6,                 //ATTN key
    VK_CRSEL = 0xF7,                //CrSel key
    VK_EXSEL = 0xF8,                //ExSel key
    VK_EREOF = 0xF9,                //EOF erase key
    VK_PLAY = 0xFA,                 //PLAY key
    VK_ZOOM = 0xFB,                 //ZOOM key
    VK_NONAME = 0xFC,               //Used on specific keyboards, usually for keys that have no specific name or function
    VK_PA1 = 0xFD,                  //PA1 key
    VK_OEM_CLEAR = 0xFE             //CLEAR key
};

class SerialMouseKeyboardControl
{
public:
    SerialMouseKeyboardControl();

    void setMousePosition( int x, int y );

    void mouseMove( int x, int y );

    void mouseRightPress();
    void mouseRightRelease();

    void mouseLeftPress();
    void mouseLeftRelease();

    void mouseMiddlePress();
    void mouseMiddleRelease();

    void setMouseScroll( int scroll );

    inline bool mouseLeftIsPressed(){ return left_is_pressed; }
    inline bool mouseRightIsPressed(){ return right_is_pressed; }
    inline bool mouseMiddleIsPressed(){ return middle_is_pressed; }

    void keyboardPress( Key key );
    void keyboardRelease( Key key );

    void keyboardPress( char key );
    void keyboardRelease( char key );

    void keyboardPrintln( const char c[] );
    void keyboardPrint( const char c[] );

private:
    bool left_is_pressed;
    bool right_is_pressed;
    bool middle_is_pressed;

    void sendKeys( bool new_line, const char str[ ] );

};

#endif
